import os from 'node:os';
import path from 'node:path';
import pLimit from 'p-limit';
import { initMatcher, outputLog, getResolvedRoot, getResolvedPublicDir, } from './utils.js';
import { CacheDb } from './db.js';
const defaultSettings = {
    cacheDir: 'node_modules/.imagemin',
    expireDuration: 10 * 24 * 60 * 60,
    countToExpire: 10,
    concurrency: os.cpus().length,
    plugins: {},
};
const defaultAssetSettings = {
    cachebuster: false,
    useCrc: false,
    preventOverwrite: false,
};
const defaultPublicSettings = {
    preventDefault: false,
    preventOverwrite: false,
};
export function initConfig(userSettings) {
    const settings = {
        ...defaultSettings,
        ...userSettings,
        asset: { ...defaultAssetSettings, ...userSettings.asset },
        public: { ...defaultPublicSettings, ...userSettings.public },
    };
    settings.cacheDir = path.normalize(settings.cacheDir);
    return settings;
}
export function initContext(config) {
    const ctx = {
        config,
        outputLog,
        publicDir: '',
        limit: pLimit(config.concurrency),
        cacheDb: new CacheDb(path.join(config.cacheDir, 'cache.db'), config.expireDuration, config.countToExpire),
        excludeMatcher: initMatcher(config.exclude),
        targetExtentions: new Set(),
        assetTargets: new Set(),
        assetHashMap: new Map(),
        publicTargets: new Set(),
        publicHashMap: new Map(),
    };
    if (config.plugins.pngquant !== false)
        ctx.targetExtentions.add('.png');
    if (config.plugins.optipng !== false)
        ctx.targetExtentions.add('.png');
    if (config.plugins.mozjpeg !== false)
        ctx.targetExtentions.add('.jpg').add('.jpeg');
    if (config.plugins.svgo !== false)
        ctx.targetExtentions.add('.svg');
    return ctx;
}
export function updateViteUserConfig(ctx, config, viteUserConfig) {
    const root = getResolvedRoot(viteUserConfig.root);
    ctx.publicDir = getResolvedPublicDir(viteUserConfig.publicDir, root);
    if (config.public.preventDefault)
        viteUserConfig.publicDir = false;
    return;
}
export function updateConfig(config, viteConfig) {
    if (Array.isArray(viteConfig.build.rollupOptions.output)) {
        config.asset.useCrc = true;
    }
    else if (viteConfig.build.rollupOptions.output?.assetFileNames &&
        typeof viteConfig.build.rollupOptions.output.assetFileNames === 'string' &&
        !viteConfig.build.rollupOptions.output.assetFileNames.includes('[hash]')) {
        config.asset.useCrc = true;
    }
    if (viteConfig.build.emptyOutDir) {
        config.asset.preventOverwrite = false;
        config.public.preventOverwrite = false;
    }
}
