import Loki from 'lokijs';
class CacheDb {
    #db;
    #coll;
    #expireDuration;
    #countToExpire;
    constructor(filename, expireDuration, countToExpire) {
        this.#db = new Loki(filename, {
            autoload: true,
            autoloadCallback: () => {
                this.#coll =
                    this.#db.getCollection('cache') ||
                        this.#db.addCollection('cache', {
                            unique: ['fileName'],
                            indices: ['fileName'],
                        });
            },
            autosave: true,
            autosaveInterval: 4000,
        });
        this.#expireDuration = expireDuration;
        this.#countToExpire = countToExpire;
    }
    countdown() {
        this.#coll.updateWhere(() => true, (obj) => {
            obj.expirationCountdown = obj.expirationCountdown - 1;
            return obj;
        });
    }
    renewAsset(targets, now = new Date().getTime()) {
        this.#coll.updateWhere((data) => targets.has(data.fileName), (obj) => {
            obj.expirationCountdown = this.#countToExpire;
            obj.lastProcessed = now;
            return obj;
        });
    }
    insertAsset(targets, now = new Date().getTime()) {
        for (const fileName of targets) {
            if (this.#coll.findOne({ fileName }))
                continue;
            this.#coll.insert({
                fileName,
                expirationCountdown: this.#countToExpire,
                lastProcessed: now,
            });
        }
    }
    getData(fileName) {
        return this.#coll.findOne({ fileName });
    }
    upsertPublic(fileName, checksum, now = new Date().getTime()) {
        const result = this.#coll.findOne({ fileName });
        if (result) {
            this.#coll
                .chain()
                .find({ fileName })
                .update((obj) => {
                obj.expirationCountdown = this.#countToExpire;
                obj.lastProcessed = now;
                obj.checksum = checksum;
                return obj;
            });
        }
        else {
            this.#coll.insert({
                fileName,
                expirationCountdown: this.#countToExpire,
                lastProcessed: now,
                checksum,
            });
        }
    }
    getExpired(now = new Date().getTime()) {
        return this.#coll.find({
            expirationCountdown: { $lt: 1 },
            lastProcessed: { $lt: now - this.#expireDuration * 1000 },
        });
    }
    removeExpired(now = new Date().getTime()) {
        this.#coll.removeWhere((data) => {
            if (data.expirationCountdown < 1 &&
                data.lastProcessed < now - this.#expireDuration * 1000)
                return true;
            return false;
        });
    }
    close() {
        this.#db.close();
    }
}
export { CacheDb };
